﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclIrDA;
using wclCommon;

namespace IrDADiscovery
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclIrDAReceiver wclIrDAReceiver;
        private wclPowerEventsMonitor FPowerMonitor;

        internal class DeviceItem
        {
            public String Address { get; set; }
            public String Name { get; set; }
            public String Hints { get; set; }
            public String CharSet { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclIrDAReceiver = new wclIrDAReceiver();
            wclIrDAReceiver.OnChange += new EventHandler(wclIrDAReceiver_OnChange);
            wclIrDAReceiver.OnDevicesFound += new wclIrDADevicesFoundEvent(wclIrDAReceiver_OnDevicesFound);

            FPowerMonitor = new wclPowerEventsMonitor();
            FPowerMonitor.OnPowerStateChanged += new wclPowerStateChangedEvent(FPowerMonitor_OnPowerStateChanged);
            FPowerMonitor.Open();

            cbClassName.SelectedIndex = 0;
            cbClassName_SelectionChanged(cbClassName, null);
        }

        void FPowerMonitor_OnPowerStateChanged(object Sender, wclPowerState State)
        {
            switch (State)
            {
                case wclPowerState.psResumeAutomatic:
                    lbEvents.Items.Add("Power state: psResumeAutomatic");
                    break;

                case wclPowerState.psResume:
                    lbEvents.Items.Add("Power state: psResume");
                    break;

                case wclPowerState.psSuspend:
                    wclIrDAReceiver.StopLazyDiscovering();
                    wclIrDAReceiver.Unsubscribe();

                    lbEvents.Items.Add("Power state: psSuspend");
                    break;

                case wclPowerState.psUnknown:
                    lbEvents.Items.Add("Power state: psUnknown");
                    break;
            }
        }

        private void wclIrDAReceiver_OnChange(object sender, EventArgs e)
        {
            if (wclIrDAReceiver.Available)
                lbEvents.Items.Add("Hardware changed. IrDA available.");
            else
                lbEvents.Items.Add("Hardware changed. IrDA NOT available.");
        }

        private void wclIrDAReceiver_OnDevicesFound(object Sender, wclIrDADevice Device)
        {
            lbEvents.Items.Add("Device found: " + Device.Address.ToString("X8") + "; " + Device.Name);
        }

        private void cbClassName_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            cbAttributeName.Items.Clear();

            switch (cbClassName.SelectedIndex)
            {
                case 0:
                    cbAttributeName.Items.Add("DeviceName");
                    cbAttributeName.Items.Add("IrLMPSupport");
                    break;
                case 1:
                    cbAttributeName.Items.Add("Parameters");
                    cbAttributeName.Items.Add("IrDA:IrLMP:LsapSel");
                    cbAttributeName.Items.Add("IrDA:TinyTP:LsapSel");
                    cbAttributeName.Items.Add("IrDA:InstanceName");
                    break;
                case 2:
                    cbAttributeName.Items.Add("IrDA:TinyTP:LsapSel");
                    break;
                case 3:
                    cbAttributeName.Items.Add("IrDA:IrLMP:LsapSel");
                    break;
                case 4:
                    cbAttributeName.Items.Add("Integer value");
                    cbAttributeName.Items.Add("Octet Sequence");
                    cbAttributeName.Items.Add("User string");
                    break;
            }

            cbAttributeName.SelectedIndex = 0;
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclIrDAReceiver.StopLazyDiscovering();
            wclIrDAReceiver.Unsubscribe();

            FPowerMonitor.Close();
        }

        private void btDiscover_Click(object sender, RoutedEventArgs e)
        {
            lvDevices.Items.Clear();
  
            wclIrDADevice[] Devices = null;
            Int32 Res = wclIrDAReceiver.Discover(out Devices);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            else
                if (Devices == null || Devices.Length == 0)
                    MessageBox.Show("Nothing found");
                else
                    for (Int32 i = 0; i < Devices.Length; i++)
                    {
                        wclIrDADevice Device = Devices[i];

                        lvDevices.Items.Add( new DeviceItem {
                            Address = Device.Address.ToString("X8"),
                            Name = Device.Name,
                            Hints = Device.Hints.ToString(),
                            CharSet = Device.CharSet.ToString()
                        });
                    }
        }

        private void btSubscribe_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAReceiver.Subscribe();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btUnsubscribe_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAReceiver.Unsubscribe();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btStart_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAReceiver.StartLazyDiscovering();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btStop_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAReceiver.StopLazyDiscovering();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btQuery_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                wclIrDAAttribute Attrib;
                Int32 Res = wclIrDAReceiver.QueryAttribute(Convert.ToUInt32(((DeviceItem)lvDevices.SelectedItems[0]).Address, 16),
                    cbClassName.SelectedItem.ToString(), cbAttributeName.SelectedItem.ToString(),
                    out Attrib);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
                else
                {
                    switch (Attrib.ValueType)
                    {
                        case wclIrDAAttributeType.atInteger:
                            MessageBox.Show("Integer: " + Attrib.IntegerValue.ToString());
                            break;

                        case wclIrDAAttributeType.atSequence:
                            String Str = "";
                            if (Attrib.SequenceValue != null)
                            {
                                if (Attrib.SequenceValue.Length > 0)
                                    for (int i = 0; i < Attrib.SequenceValue.Length; i++)
                                        Str = Str + Attrib.SequenceValue[i].ToString("X2");

                                Attrib.SequenceValue = null;
                            }

                            MessageBox.Show("Sequence: " + Str);
                            break;

                        case wclIrDAAttributeType.atString:
                            MessageBox.Show("String: " + Attrib.StringValue);
                            break;
                    }
                }
            }
        }
    }
}
