﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;

using wclCommon;
using wclCommunication;
using wclIrDA;

namespace OppServer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclIrDAServer wclIrDAServer;
        private String FFileName;

        private void Trace(String Msg)
        {
            lbLog.Items.Add(Msg);
        }

        public MainWindow()
        {
            InitializeComponent();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbLog.Items.Clear();
        }

        private void btListen_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAServer.Listen();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btClose_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAServer.Close();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            // Simple try to close the server.
            wclIrDAServer.Close();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclIrDAServer = new wclIrDAServer();
            wclIrDAServer.Service = "OBEX";
            wclIrDAServer.Mode = wclIrDAConnectionMode.cmIrComm3Wire; // Required for OBEX connection!

            wclIrDAServer.OnListen += new EventHandler(wclIrDAServer_OnListen);
            wclIrDAServer.OnClosed += new wclClientConnectionDisconnectEvent(wclIrDAServer_OnClosed);
            wclIrDAServer.OnDisconnect += new wclIrDAServerDisconnectEvent(wclIrDAServer_OnDisconnect);
            wclIrDAServer.OnDestroyProcessor += new wclServerConnectionProcessorEvent(wclIrDAServer_OnDestroyProcessor);
            wclIrDAServer.OnConnect += new wclIrDAServerConnectEvent(wclIrDAServer_OnConnect);
            wclIrDAServer.OnCreateProcessor += new wclServerConnectionProcessorEvent(wclIrDAServer_OnCreateProcessor);
        }

        void wclIrDAServer_OnListen(object sender, EventArgs e)
        {
            Trace("Server listening");
        }

        void wclIrDAServer_OnClosed(object Sender, int Reason)
        {
            Trace("Server closed. Reason:: 0x" + Reason.ToString("X8"));
        }

        void wclIrDAServer_OnDisconnect(object Sender, wclIrDAServerClientConnection Client, int Reason)
        {
            Trace("Client " + Client.Address.ToString("X8") + " disconnected. Reason: 0x" +
                Reason.ToString("X8"));
        }

        void wclIrDAServer_OnDestroyProcessor(object Sender, wclServerClientDataConnection Connection)
        {
            Trace("Data processor for client " +
                ((wclIrDAServerClientConnection)Connection).Address.ToString("X8") +
                " is destroing");
            Connection.Processor.Dispose();
        }

        void wclIrDAServer_OnConnect(object Sender, wclIrDAServerClientConnection Client, int Error)
        {
            if (Error == wclErrors.WCL_E_SUCCESS)
                Trace("Client " + Client.Address.ToString("X8") + " connected");
            else
                Trace("Client " + Client.Address.ToString("X8") + " connect failed: 0x" + Error.ToString("X8"));
        }

        void wclIrDAServer_OnCreateProcessor(object Sender, wclServerClientDataConnection Connection)
        {
            // Here we must create OBEX OPP server data processor.
            wclObexOppServer Proc = new wclObexOppServer(Connection);
            Proc.OnConnect += new wclObexOppServerConnectEvent(Proc_OnConnect);
            Proc.OnDisconnected += new wclObexServerClientDisconnectedEvent(Proc_OnDisconnected);
            Proc.OnPutBegin += new wclObexServerClientPutBeginEvent(Proc_OnPutBegin);
            Proc.OnPutCompleted += new wclObexServerClientPutCompletedEvent(Proc_OnPutCompleted);
            Proc.OnPutProgress += new wclObexServerClientOperationProgressEvent(Proc_OnPutProgress);
        }

        void Proc_OnConnect(object Sender, string Description)
        {
            wclObexOppServer Proc = (wclObexOppServer)Sender;
            Int64 Address = ((wclIrDAServerClientConnection)Proc.Connection).Address;
            Boolean Accept = MessageBox.Show("Client " + Address.ToString("X8") +
                " requsts for connection: " + Description + ". Accept?", "Accept connection",
                MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes;

            Int32 Res;
            if (Accept)
                Res = Proc.Accept("Accepted by user");
            else
                Res = Proc.Reject("Rejected by user");

            if (Res != wclErrors.WCL_E_SUCCESS)
            {
                MessageBox.Show("Operation failed: 0x" + Res.ToString("X8"));
                // Disconnect client!
                Proc.Connection.Disconnect();
            }
        }

        void Proc_OnDisconnected(object Sender, int Reason, string Description)
        {
            wclObexOppServer Proc = (wclObexOppServer)Sender;
            Int64 Address = ((wclIrDAServerClientConnection)Proc.Connection).Address;

            Trace("Client " + Address.ToString("X8") + " closed OBEX session with reason: 0x" + Reason.ToString("X8") +
                ". Description: " + Description);
        }

        void Proc_OnPutBegin(object Sender, string Name, string Description, string Mime, uint Length, out bool Accept)
        {
            wclObexOppServer Proc = (wclObexOppServer)Sender;
            Int64 Address = ((wclIrDAServerClientConnection)Proc.Connection).Address;
            String Str = "Client " + Address.ToString("X8") + " send file " + Name +
                ". Accept?";
            Accept = MessageBox.Show(Str, "Accept file", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes;

            if (Accept)
                FFileName = Name;
        }

        void Proc_OnPutCompleted(object Sender, int Error, System.IO.Stream Stream, out bool Accept)
        {
            wclObexOppServer Proc = (wclObexOppServer)Sender;
            Int64 Address = ((wclIrDAServerClientConnection)Proc.Connection).Address;
            if (Error == wclErrors.WCL_E_SUCCESS)
            {
                String Str = "Client " + Address.ToString("X8") + " completed sent file. Accept?";
                Accept = MessageBox.Show(Str, "Accept File", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes;

                if (Accept)
                {
                    Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
                    dlg.FileName = FFileName;
                    if (dlg.ShowDialog() == true)
                    {
                        Stream.Seek(0, SeekOrigin.Begin);
                        FileStream fs = new FileStream(dlg.FileName, FileMode.CreateNew, FileAccess.Write);
                        Byte[] Bytes = new Byte[Stream.Length];
                        Stream.Read(Bytes, 0, (int)Stream.Length);
                        fs.Write(Bytes, 0, Bytes.Length);
                        fs.Close();
                    }
                }
            }
            else
            {
                Trace("Send file error: 0x" + Error.ToString("X8"));
                Accept = false;
            }
        }

        void Proc_OnPutProgress(object Sender, uint Position, uint Length, out bool Continue)
        {
            Trace("Sending file: " + Position.ToString() + " from " + Length.ToString());
            // Do not foget to set Continue flag!
            Continue = true;
        }
    }
}
